package ca.canadapost.cpcdp.platforms;

import ca.canadapost.cpcdp.platforms.stub.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.platforms.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetMerchantRegistrationToken Canada Post service.
 * <p>
 * The GetMerchantRegistrationToken service returns a unique registration token that is used
 * to launch a merchant into the Canada Post sign-up process.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetMerchantRegistrationTokenClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try {
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_Platforms_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");

            MerchantRegistrationService service = new MerchantRegistrationService();
            MerchantRegistrationPortType port = service.getMerchantRegistrationPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/ot/soap/merchant/registration");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetMerchantRegistrationTokenRequest request = new GetMerchantRegistrationTokenRequest();
            request.setLocale("FR");

            //execute request
            GetMerchantRegistrationTokenResponse response = port.getMerchantRegistrationToken(request);

            //retrieve values from response
            if(response.getMessages() == null){
                System.out.println("Token Id: " + response.getToken().getTokenId());
            } else {
                for(MessagesType.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
