package ca.canadapost.cpcdp.rating;

import ca.canadapost.cpcdp.rating.stub.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.rating.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetOption Canada Post service.
 * <p>
 * The GetOption service  returns information about a given add-on option such
 * as how it is used and whether it requires or conflicts with other options.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetOptionClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
            // Your username, password and customer number are imported from the following file: CPCWS_SOAP_Rating_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");

            RatingService service = new RatingService();
            RatingPortType port = service.getRatingPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/rating/v4");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //create request
            GetOptionRequest request = new GetOptionRequest();
            request.setLocale("FR");
            request.setOptionCode("DC");

            //execute request
            GetOptionResponse response = port.getOption(request);

            //retrieve values from response
            if(response.getMessages() == null){
                System.out.println("Option Code: " + response.getOption().getOptionCode());
                System.out.println("Option Name: " + response.getOption().getOptionName());
                System.out.println("Option Class: " + response.getOption().getOptionClass());
                System.out.println("Print on Label: " + response.getOption().isPrintsOnLabel());
                System.out.println("Qualifier Required: " + response.getOption().isQualifierRequired());
            } else {
                // Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
