package ca.canadapost.cpcdp.rating.util;

import org.apache.cxf.ws.security.wss4j.WSS4JOutInterceptor;
import org.apache.ws.security.WSConstants;
import org.apache.ws.security.WSPasswordCallback;
import org.apache.ws.security.handler.WSHandlerConstants;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * Helper class for client examples.
 */
public class ClientUtil {
    /**
     * Read values from properties file.
     *
     * @param filename
     * @return Map<String, String>: key-value pairs from supplied properties file
     * @throws Exception
     */
    public static Map<String, String> getUserProperties(String filename) throws Exception{
        Properties properties = new Properties();
        FileInputStream propInputStream = null;
        Map<String, String> userProperties = new HashMap<String, String>();

        try {
            propInputStream = new FileInputStream(filename);
            properties.load(propInputStream);

            for (String key : properties.stringPropertyNames()) {
                String value = properties.getProperty(key);
                userProperties.put(key, value);
            }
        } finally {
            if(propInputStream != null){
                try {
                    propInputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        return userProperties;
    }


    /**
     * Create a WSS4J out interceptor for adding a UsernameToken to a soap header.
     *
     * @param username
     * @param password
     * @return WSS4JOutInterceptor
     */
    public static WSS4JOutInterceptor getUsernameTokenInterceptor(String username, String password) {
        Map props = new HashMap();
        props.put(WSHandlerConstants.ACTION, WSHandlerConstants.USERNAME_TOKEN);
        props.put(WSHandlerConstants.USER, username);
        props.put(WSHandlerConstants.PASSWORD_TYPE, WSConstants.PW_TEXT);
        props.put(WSHandlerConstants.PW_CALLBACK_REF, new PasswordCallback(password));

        return new WSS4JOutInterceptor(props);
    }


    /**
     * Used by getUsernameTokenInterceptor for setting the password
     */
    public static class PasswordCallback implements CallbackHandler {
        private final String password;

        private PasswordCallback(String password) {
            this.password = password;
        }

        @Override
        public void handle(Callback[] callbacks) throws IOException, UnsupportedCallbackException {
            WSPasswordCallback pc = (WSPasswordCallback) callbacks[0];
            pc.setPassword(password);
        }
    }
}
