package ca.canadapost.cpcdp.returns;

import ca.canadapost.cpcdp.returns.stub.openreturn.*;
import org.apache.commons.codec.binary.Base64;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.returns.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.io.FileOutputStream;
import java.util.Map;

/**
 * Sample code for the RetrieveNextOpenReturnArtifact Canada Post service.
 * <p>
 * The RetrieveNextOpenReturnArtifact service is used to retrieve the PDF image or ZPL output
 * of a return label artifact that was pre-established using Create Open Return Template.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class RetrieveNextOpenReturnArtifactClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
            // Your username, password and customer number are imported from the following file: CPCWS_SOAP_Returns_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            OpenReturnService service = new OpenReturnService();
            OpenReturnPortType port = service.getOpenReturnPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/openreturn/v2");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //create request
            RetrieveNextOpenReturnArtifactRequest request = new RetrieveNextOpenReturnArtifactRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setTemplateId("816111401403204607");

            //execute response
            RetrieveNextOpenReturnArtifactResponse response = port.retrieveNextOpenReturnArtifact(request);

            // Retrieve values from response object
            if(response.getMessages() == null){
                // Decoding base64 certificate to a file
                String fileName = "openReturnArtifact.pdf";
                String encodedImage = response.getArtifactData().getImage();
                System.out.println("Base64 Encoded: " + encodedImage);
                System.out.println("Mime type: " + response.getArtifactData().getMimeType());
                FileOutputStream fos = new FileOutputStream(fileName);
                Base64 decoder = new Base64();
                byte[] decodedBytes = decoder.decode(encodedImage.getBytes());
                fos.write(decodedBytes);
                fos.close();
                System.out.println("Decoding to " + System.getProperty("user.dir") + System.getProperty("file.separator") + fileName);
            } else {
                // Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
