package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.shipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetGroups Canada Post service.
 * <p>
 * The GetGroups service is used to retrieve the list of unique group-id
 * codes associated with a customer (or mailed-on-behalf-of customer) for which
 * there are shipments created that have not yet been transmitted.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetGroupsClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ShipmentService service = new ShipmentService();
            ShipmentPortType port = service.getShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetGroupsRequest request = new GetGroupsRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);

            //execute request
            GetGroupsResponse response = port.getGroups(request);

            //retrieve values from response
            if(response.getMessages() == null){
                if(response.getGroups() != null) {
                    for(String id : response.getGroups().getGroupId()){
                        System.out.println("Group Id: " + id);
                    }
                } else {
                    System.out.println("No groups returned.");
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
