package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.shipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetShipment Canada Post service.
 * <p>
 * The GetShipment service is used to retrieve information related to a previously
 * created shipment. This may be useful for recovery from a communication or other
 * error and/or reporting on all outstanding shipments.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetShipmentClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ShipmentService service = new ShipmentService();
            ShipmentPortType port = service.getShipmentPort();

            //set endpoint
            BindingProvider bindingProvider = (BindingProvider) port;
            bindingProvider.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //log request/response
            //client.getInInterceptors().add(new LoggingInInterceptor());
            //client.getOutInterceptors().add(new LoggingOutInterceptor());

            //create request
            ObjectFactory of = new ObjectFactory();

            GetShipmentRequest request = of.createGetShipmentRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setShipmentId("522121392859903208");

            //execute request
            GetShipmentResponse response = port.getShipment(request);

            //retrieve values from response
            if(response.getMessages() == null){
                ShipmentInfoType shipmentInfoType = response.getShipmentInfo();
                System.out.println("Shipping Id: " + shipmentInfoType.getShipmentId());
                System.out.println("Tracking Pin: " + shipmentInfoType.getTrackingPin());

                for(ArtifactType a : shipmentInfoType.getArtifacts().getArtifact()){
                    System.out.println("Artifact Type: " + a.getType());
                    System.out.println("Artifact Id: " + a.getArtifactId());
                    System.out.println("Page Index: " + a.getPageIndex());
                    System.out.println();
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
