package ca.canadapost.cpcdp.tracking;

import ca.canadapost.cpcdp.tracking.stub.*;
import org.apache.commons.codec.binary.Base64;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import ca.canadapost.cpcdp.tracking.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.io.FileOutputStream;
import java.util.Map;

/**
 * Sample code for the GetDeliveryConfirmationCertificate Canada Post service.
 * <p>
 * If the parcel has been delivered, the GetDeliveryConfirmationCertificate  service
 * returns a signature image captured at delivery of the parcel if available. (The
 * parcel is identified using a PIN only).
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetDeliveryConfirmationCertificateClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try {
            // Your username and password are imported from the following file: CPCWS_SOAP_Tracking_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");

            TrackingService service = new TrackingService();
            TrackingPortType port = service.getTrackingPort();


            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            //ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/vis/soap/track");
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/vis/soap/track");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //log request/response
            //client.getInInterceptors().add(new LoggingInInterceptor());
            //client.getOutInterceptors().add(new LoggingOutInterceptor());

            //build request
            GetDeliveryConfirmationCertificateRequest request = new GetDeliveryConfirmationCertificateRequest();
            request.setLocale("FR");
            request.setPin("1371134583769923");

            //execute request
            GetDeliveryConfirmationCertificateResponse response = port.getDeliveryConfirmationCertificate(request);

            //retrieve values from response
            if(response.getMessages() == null){
                // Decoding base64 certificate to a file
                String fileName = response.getDeliveryConfirmationCertificate().getFilename();
                String encodedImage = response.getDeliveryConfirmationCertificate().getImage();
                System.out.println("Base64 Encoded: " + encodedImage);
                System.out.println("File name: " + fileName);
                System.out.println("Mime type: " + response.getDeliveryConfirmationCertificate().getMimeType());
                FileOutputStream fos = new FileOutputStream(fileName);
                Base64 decoder = new Base64();
                byte[] decodedBytes = decoder.decode(encodedImage.getBytes());
                fos.write(decodedBytes);
                fos.close();
                System.out.println("Decoding to " + System.getProperty("user.dir") + System.getProperty("file.separator") + fileName);
            } else {
                // Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
