package ca.canadapost.cpcdp.tracking;

import ca.canadapost.cpcdp.tracking.stub.*;

import ca.canadapost.cpcdp.tracking.stub.Messages;
import org.apache.cxf.binding.soap.SOAPBindingUtil;
import org.apache.cxf.binding.soap.SoapBindingFactory;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import org.apache.cxf.jaxws.JaxWsProxyFactoryBean;
import ca.canadapost.cpcdp.tracking.util.ClientUtil;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetTrackingSummary Canada Post service.
 * <p>
 * The GetTrackingSummary service returns the most recent/significant event for a 
 * parcel. If it has been delivered, the delivery details are returned. (The parcel 
 * is identified using PIN, DNC or the other reference parameters).
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */
public class GetTrackingSummaryClient {
    	
    // Main class for local testing only
    public static void main(String[] args) {    	
        try {
        	// Your username and password are imported from the following file: CPCWS_SOAP_Tracking_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");

            TrackingService service = new TrackingService();
            TrackingPortType port = service.getTrackingPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/vis/soap/track");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //client.getInInterceptors().add(new LoggingInInterceptor());
            //client.getOutInterceptors().add(new LoggingOutInterceptor());

            //build request
            GetTrackingSummaryRequest request = new GetTrackingSummaryRequest();
            request.setLocale("FR");

            //PIN request
            request.setPin("1681334332936901");

            //DNC request
            //request.setDnc("315052413796541");

            //Reference Search request
            //ReferenceCriteria referenceCriteria = new ReferenceCriteria();
            //referenceCriteria.setDestinationPostalCode("K2H7X3");
            //referenceCriteria.setReferenceNumber("DIA101");
            //referenceCriteria.setMailingDateFrom(DatatypeFactory.newInstance().newXMLGregorianCalendar("2014-03-04"));
            //referenceCriteria.setMailingDateTo(DatatypeFactory.newInstance().newXMLGregorianCalendar("2014-03-06"));
            //request.setReferenceCriteria(referenceCriteria);

            //execute request
            GetTrackingSummaryResponse response = port.getTrackingSummary(request);

            //retrieve values from response
            if(response.getMessages() == null){
                for(PinSummary pinSummary : response.getTrackingSummary().getPinSummary()){
                    System.out.println("PIN Number: " + pinSummary.getPin());
                    System.out.println("Mailed On Date: " + pinSummary.getMailedOnDate());
                    System.out.println("Event Description: " + pinSummary.getEventDescription());
                    System.out.println();
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }

        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
